package com.qhtx.eta.domain.api.proxy;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.qhtx.eta.domain.enums.ExcludedField;
import com.qhtx.eta.common.eunms.ErrorEnum;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.lang.reflect.Field;
import java.util.*;

@Slf4j
@Data
public abstract class ApiParameterProxy<T> {

    protected static final ThreadLocal<Object> THREAD_LOCAL_DATA = new ThreadLocal<>();

    public void setData(Object data) {
        THREAD_LOCAL_DATA.set(data);
    }

    /**
     * 获取线程局部变量data
     */
    public Object getData() {
        return THREAD_LOCAL_DATA.get();
    }


    /**
     * 代理方法
     */
    protected abstract T acceptParam(Object data);

    public Map<String, Object> parameters(Object data) {
        if (data == null) {
            return null;
        }
        setData(acceptParam(data));
        return afterExecute(getData());
    }

    /**
     * 将参数转成map
     */
    private Map<String, Object> afterExecute(Object data) {
        Class<?> clazz = data.getClass();
        Field[] fields = clazz.getDeclaredFields();
        fields = Arrays.stream(fields).filter(field -> Arrays.stream(ExcludedField.values()).noneMatch(excludedField -> field.getName().equals(excludedField.getFieldName()))).toArray(Field[]::new);
        Map<String, Object> params = new HashMap<>(fields.length);
        for (Field field : fields) {
            field.setAccessible(true);
            try {
                Object obj = field.get(data);
                if (obj != null) {
                    JsonProperty prop = field.getAnnotation(JsonProperty.class);
                    String name = prop != null ? (StringUtils.isEmpty(prop.value()) ? field.getName() : prop.value()) : field.getName();
                    if (obj instanceof List) {
                        List<Map<String, Object>> paramsList = new ArrayList<>();
                        for (Object o : (List<?>) obj) {
                            paramsList.add(afterExecute(o));
                        }
                        params.put(name, paramsList);
                    } else {
                        params.put(name, obj);
                    }
                }
            } catch (IllegalAccessException e) {
                log.error("{},类:{}-{}", ErrorEnum.BEAN_COVERT_TO_MAP_ERROR.getMsg(), data.getClass().getName(), JSONObject.toJSONString(data));
            }
        }
        THREAD_LOCAL_DATA.remove();
        return params;
    }
}
