package com.qhtx.eta.domain.config;

import com.alibaba.nacos.common.utils.StringUtils;
import org.redisson.Redisson;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.redisson.config.Config;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.Arrays;
import java.util.List;

@Configuration
public class RedissonConfig {

    @Value("${redisson.address}")
    private String address;
    @Value("${redisson.password}")
    private String password;

    @Value("${redisson.database}")
    private int database;

    @Bean
    public RedissonClient redissonClient() {
        Config config = new Config();
        List<String> hosts = Arrays.asList(address.split(";"));
        if (hosts.size() == 1) {
            config.useSingleServer().setAddress("redis://"+address).setDatabase(database);
            if (StringUtils.isNotBlank(password)) {
                config.useSingleServer().setAddress("redis://"+address).setPassword(password).setDatabase(database);
            }
        } else {
            if (StringUtils.isNotBlank(password)) {
                config.useClusterServers().setPassword(password);
            }
            hosts.forEach(h -> config.useClusterServers().addNodeAddress("redis://"+h));
        }
        return Redisson.create(config);
    }

}