package com.qhtx.eta.domain.service.impl;

import com.alibaba.druid.util.StringUtils;
import com.qhtx.eta.common.constant.ETAConstants;
import com.qhtx.eta.common.eunms.ErrorEnum;
import com.qhtx.eta.common.exception.ETAException;
import com.qhtx.eta.domain.config.ETAConfigProperties;
import com.qhtx.eta.domain.convert.DWIndexDTOConverter;
import com.qhtx.eta.domain.convert.DWIndexDataDTOConverter;
import com.qhtx.eta.domain.convert.DWIndexFrameDTOConverter;
import com.qhtx.eta.domain.entity.DWIndexDTO;
import com.qhtx.eta.domain.entity.DWIndexDataDTO;
import com.qhtx.eta.domain.entity.DWIndexFrameDTO;
import com.qhtx.eta.domain.service.DWIndexFrameService;
import com.qhtx.eta.domain.utils.RedisUtils;
import com.qhtx.eta.infra.annotation.UseDataSource;
import com.qhtx.eta.infra.datasource.DataSourceContextHolder;
import com.qhtx.eta.infra.entity.dw.TDampDwLinkDetail;
import com.qhtx.eta.infra.entity.dw.*;
import com.qhtx.eta.infra.mapper.DWIndexFrameDao;
import com.qhtx.eta.infra.service.*;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import com.qhtx.eta.common.eunms.DataSourceType;

@Service
@Slf4j
public class DWIndexFrameServiceImpl implements DWIndexFrameService {

    @Resource
    private ETAConfigProperties etaConfigProperties;
    @Resource
    private RedisUtils redisUtils;

    @Resource
    private IndexFrameService indexFrameService;

    @Resource
    private TransactionTemplate transactionTemplate;

    @Resource
    private EtaApiClassifyService etaApiClassifyService;

    @Resource
    private TEtaIndexPushTaskService etaIndexPushTaskService;

    @Resource
    private TDampDwIndexDataService tDampDwIndexDataService;

    @Resource
    private TDampDwIndexLinkService tDampDwIndexLinkService;

    @Resource
    private TDampDwLinkDetailService tDampDwLinkDetailService;

    @Resource
    private TDampDwIndexService tDampDwIndexService;
    @Resource(name = "IndexDataThreadPool")
    private ThreadPoolExecutor indexDataThreadPool;

    @Override
    public void syncIndexFrame() {

    }

    @Override
    public String generateTableCode() throws ETAException {
        if (!redisUtils.exist(ETAConstants.REDIS_INDEX_FRAME_TABLE_CODE_KEY)) {
            //   throw new ETAException(ErrorEnum.DW_INDEX_TABLE_CODE_GET_ERROR);
            synchronized (this) {
                if (!redisUtils.exist(ETAConstants.REDIS_INDEX_FRAME_TABLE_CODE_KEY)) {
                    String maxTableCode = indexFrameService.getMaxTableCode();
                    long maxData = maxTableCode == null ? 0L : Long.parseLong(maxTableCode.replaceAll(ETAConstants.INDEX_FRAME_TABLE_CODE_PREFIX, ""));
                    redisUtils.setNX(ETAConstants.REDIS_INDEX_FRAME_TABLE_CODE_KEY, maxData);
                }
            }
        }
        return ETAConstants.INDEX_FRAME_TABLE_CODE_PREFIX + String.format("%07d", redisUtils.add(ETAConstants.REDIS_INDEX_FRAME_TABLE_CODE_KEY, 1L));
    }

    @Override
    public void processing(List<DWIndexFrameDTO> processList) {
//        DWIndexFrame hzIndexFrame = indexFrameService.getByCnName(ETAConstants.DW_INDEX_FRAME_PARENT_NAME);
        DWIndexFrame hzIndexFrame = indexFrameService.getByCnName(etaConfigProperties.getEtaCategoryName());
        if (hzIndexFrame == null) {
            log.info("未找到根节点,不同步eta指标目录");
            return;
        }
        hzIndexFrame.setHasChild(1);
        FrameNode root = new FrameNode();
        root.setId(hzIndexFrame.getId());
        root.setName(hzIndexFrame.getCnName());
        root.setIdPath(hzIndexFrame.getFrameIdPath());
        root.setPath(hzIndexFrame.getFramePath());
        root.setPId(hzIndexFrame.getPId());
        root.setFrameLevel(hzIndexFrame.getFrameLevel());
        root.setClassifyId(0);
        root.setParentClassifyId(0);
        root.setChildren(new ArrayList<>());
        buildTree(processList, root, 1, 10);
        //建完树开始插入数据库 ，树转List
        List<DWIndexFrameDTO> dtoList = parseList(root);
        log.info("开始插入数据库{}", dtoList.size());
        for (DWIndexFrameDTO dto : dtoList) {
            ETAClassifyIndexFrameMapping mapping = etaApiClassifyService.getByClassifyId(dto.getClassifyId());
            if (mapping == null) {
                log.info("未找到对应eta分类,{}:{}", dto.getCnName(), dto.getClassifyId());
                continue;
            }
            //无法代理方法直接使用手动方法设置数据源
            DataSourceContextHolder.setDataSourceType(DataSourceType.DW);
            transactionTemplate.execute(new TransactionCallbackWithoutResult() {
                @Override
                protected void doInTransactionWithoutResult(@NotNull TransactionStatus transactionStatus) {
                    try {
                        //更新mapping已处理
                        mapping.setIndexFrameId(dto.getId());
                        etaApiClassifyService.update(mapping);
                        indexFrameService.insertOrUpdate(DWIndexFrameDTOConverter.INSTANCE.convertDTOToEntity(dto));
                    } catch (Exception e) {
                        log.warn(String.format(ErrorEnum.DATASOURCE_EXECUTE_ERROR.getMsg(), e.getMessage()));
                        throw e;
                    }
                }
            });
        }
    }

    @Override
    public DWIndex getIndexByIndexCode(String indexCode) {
        return indexFrameService.getIndexByIndexCode(indexCode);
    }

    @Override
    public DWIndexDTO addSyncIndex(String indexCode) {
        DWIndexDTO dwIndexDTO = getIndex(indexCode);
        TEtaIndexPushTask tEtaIndexPushTask = DWIndexDTOConverter.INSTANCE.convertToTEtaIndexPushTask(dwIndexDTO);
        etaIndexPushTaskService.addTask(tEtaIndexPushTask);
        return dwIndexDTO;
    }

    @Override
    public DWIndexDTO getIndex(String indexCode) {
        DWIndex dwIndex = indexFrameService.getIndexByIndexCode(indexCode);
        if (dwIndex == null) {
            log.warn("指标信息不存在,indexCode:{}", indexCode);
            throw new ETAException(ErrorEnum.DW_INDEX_NOT_FOUND);
        }
        if (StringUtils.isEmpty(dwIndex.getSourceName())) {
            dwIndex.setSourceName("钢联");
        }
        DWIndexDTO dwIndexDTO = DWIndexDTOConverter.INSTANCE.convertToDTO(dwIndex);
        dwIndexDTO.setRemark("");
        return dwIndexDTO;
    }

    @Override
    public List<DWIndexDataDTO> getIndexData(String indexCode) {
        int total = tDampDwIndexDataService.countByIndexCode(indexCode);
        List<TDampDwIndexData> allResults;
        //分块取值
        if (total > ETAConstants.ETA_PUSH_DATA_LIMIT) {
            int chunk = (total + ETAConstants.ETA_PUSH_DATA_LIMIT - 1) / ETAConstants.ETA_PUSH_DATA_LIMIT;
            List<CompletableFuture<List<TDampDwIndexData>>> futures = new ArrayList<>();
            for (int i = 0; i < chunk; i++) {
                int pars = i;
                CompletableFuture<List<TDampDwIndexData>> future = CompletableFuture.supplyAsync(() -> {
                    try {
                        return tDampDwIndexDataService.queryAllByLimit(indexCode, pars * ETAConstants.ETA_PUSH_DATA_LIMIT, ETAConstants.ETA_PUSH_DATA_LIMIT);
                    } catch (Exception e) {
                        log.error("获取指标数据失败", e);
                        throw new ETAException(ErrorEnum.DATASOURCE_EXECUTE_ERROR);
                    }
                }, indexDataThreadPool);
                futures.add(future);
            }
            // 等待所有任务完成并合并结果
            allResults = futures.stream().map(CompletableFuture::join).flatMap(List::stream).collect(Collectors.toList());
        } else {
            allResults = tDampDwIndexDataService.queryAllByLimit(indexCode, 0, total);
        }
        return allResults.stream().map(DWIndexDataDTOConverter.INSTANCE::convertToDTO).collect(Collectors.toList());
    }

    @Override
    public void insertIndex(TDampDwIndex dwIndex) {
        tDampDwIndexService.insert(dwIndex);
    }

    @Override
    public List<Long> getIndexSeq(int nums) {
        return tDampDwIndexService.getIndexSeq(nums);
    }

    @Override
    public void insertIndexLink(TDampDwIndexLink tDampDwIndexLink) {
        tDampDwIndexLinkService.insert(tDampDwIndexLink);
    }

    @Override
    public List<TDampDwLinkDetail> getlinkDataList(String linkCode) {
        return tDampDwLinkDetailService.queryByLinkCode(linkCode);
    }

    @Override
    public void batchInsertIndexData(List<TDampDwIndexData> dwLinkDetailList) {
        tDampDwIndexDataService.batchInsertIndexData(dwLinkDetailList);
    }

    private List<DWIndexFrameDTO> parseList(FrameNode node) {
        if (node == null) {
            return null;
        }
        List<DWIndexFrameDTO> result = new ArrayList<>();
        DWIndexFrameDTO dto = new DWIndexFrameDTO();
        dto.setTableCode(generateTableCode());
        dto.setId(node.getId());
        dto.setCnName(node.getName());
        dto.setFrameLevel(node.getFrameLevel());
        dto.setPId(node.getPId());
        dto.setSort(node.getSort());
        dto.setFramePath(node.getPath());
        dto.setFrameIdPath(node.getIdPath());
        dto.setClassifyId(node.getClassifyId());
        dto.setParentId(node.getParentClassifyId());
        result.add(dto);
        if (!CollectionUtils.isEmpty(node.children)) {
            dto.setHasChild(1);
            for (FrameNode subNode : node.children) {
                result.addAll(parseList(subNode));
            }
        } else {
            dto.setHasChild(0);
        }
        return result;
    }

    private void buildTree(List<DWIndexFrameDTO> list, FrameNode node, int current, int depth) {
        if (current > depth) {
            return;
        }
        for (DWIndexFrameDTO dto : list) {
            if (Objects.equals(dto.getParentId(), node.getClassifyId())) {
                String path = StringUtils.isEmpty(node.getPath().trim()) ? node.getName()  : node.getPath() + "|" + node.getName();
                String id = (dto.getId() == null || dto.getId().trim().isEmpty()) ? UUID.randomUUID().toString().replaceAll("-", "") : dto.getId();
                String idPath = StringUtils.isEmpty(node.getPath().trim()) ? node.getId() : node.getIdPath() + "|" + node.getId();
                FrameNode temp = new FrameNode();
                temp.setId(id);
                temp.setClassifyId(dto.getClassifyId());
                temp.setParentClassifyId(dto.getParentId());
                temp.setIdPath(idPath);
                temp.setClassifyId(dto.getClassifyId());
                temp.setFrameLevel(node.getFrameLevel() + 1);
                temp.setParentClassifyId(dto.getParentId());
                temp.setName(dto.getCnName());
                temp.setPId(node.getId());
                temp.setPath(path);
                temp.setSort(dto.getSort());
                temp.setChildren(new ArrayList<>());
                node.getChildren().add(temp);
                buildTree(list, temp, current + 1, depth);
            }
        }
    }

    @Data
    private static class FrameNode {
        private int frameLevel;
        private String pId;
        private String id;
        private String path;
        private String idPath;
        private String name;
        private Integer classifyId;
        private Integer parentClassifyId;
        private Integer sort;
        private List<FrameNode> children;
    }
}
