package com.qhtx.eta.domain.task;

import org.springframework.beans.factory.DisposableBean;
import org.springframework.scheduling.TaskScheduler;
import org.springframework.scheduling.config.FixedDelayTask;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Component
public class FixedDelayTaskRegistrar implements DisposableBean {

    private final Map<Runnable, ScheduledTask> scheduledTasks = new ConcurrentHashMap<>(16);

    @Resource
    private TaskScheduler taskScheduler;

    @Override
    public void destroy() throws Exception {
        for (ScheduledTask task : this.scheduledTasks.values()) {
            task.cancel();
        }
        this.scheduledTasks.clear();
    }

    public TaskScheduler getScheduler() {
        return this.taskScheduler;
    }

    public void addFixedDelayTask(Runnable task, long interval, long initialDelay) {
        addFixedDelayTask(new FixedDelayTask(task, interval, initialDelay));
    }

    public void addFixedDelayTask(FixedDelayTask fixedDelayTask) {
        if (fixedDelayTask != null) {
            Runnable task = fixedDelayTask.getRunnable();
            if (this.scheduledTasks.containsKey(task)) {
                removeFixedDelayTask(task);
            }
            this.scheduledTasks.put(task, scheduleFixedDelayTask(fixedDelayTask));
        }
    }

    public void removeFixedDelayTask(Runnable task) {
        ScheduledTask scheduledTask = this.scheduledTasks.remove(task);
        if (scheduledTask != null) scheduledTask.cancel();
    }

    public ScheduledTask scheduleFixedDelayTask(FixedDelayTask fixedDelayTask) {
        ScheduledTask scheduledTask = new ScheduledTask();
        Date currentTime = fixedDelayTask.getInitialDelay() > 0 ? new Date(System.currentTimeMillis() + fixedDelayTask.getInitialDelay()) : new Date();
        scheduledTask.future = this.taskScheduler.scheduleWithFixedDelay(fixedDelayTask.getRunnable(), currentTime, fixedDelayTask.getInterval());
        return scheduledTask;
    }

}
