package com.qhtx.eta.domain.task;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.ReflectionUtils;
import org.springframework.util.StringUtils;

import java.lang.reflect.Method;
import java.util.Objects;

public class SchedulingRunnable implements Runnable {

    private static final Logger logger = LoggerFactory.getLogger(SchedulingRunnable.class);

    private String beanName;

    private String methodName;

    private Object target;

    public SchedulingRunnable(String beanName, String methodName) {
        this(beanName, methodName, null);
    }

    public SchedulingRunnable(String beanName, String methodName, Object target) {
        this.beanName = beanName;
        this.methodName = methodName;
        this.target = target;
    }


    @Override
    public void run() {
        logger.info("定时任务开始执行 - bean：{}，方法：{}", beanName, methodName);
        long startTime = System.currentTimeMillis();
        try {
            Method method = target.getClass().getDeclaredMethod(methodName);
            ReflectionUtils.makeAccessible(method);
            method.invoke(target);
        } catch (Exception ex) {
            logger.error(String.format("定时任务执行异常 - bean：%s，方法：%s", beanName, methodName), ex);
        }
        long times = System.currentTimeMillis() - startTime;
        logger.info("定时任务执行结束 - bean：{}，方法：{}，耗时：{} 毫秒", beanName, methodName, times);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        SchedulingRunnable that = (SchedulingRunnable) o;
        return beanName.equals(that.beanName) && methodName.equals(that.methodName);
    }

    @Override
    public int hashCode() {
        return Objects.hash(beanName, methodName);
    }
}