package com.qhtx.eta.domain.task;

import com.alibaba.cloud.nacos.NacosConfigManager;
import com.alibaba.cloud.nacos.NacosConfigProperties;
import com.alibaba.nacos.api.config.ConfigChangeEvent;
import com.alibaba.nacos.api.config.ConfigChangeItem;
import com.alibaba.nacos.api.exception.NacosException;
import com.alibaba.nacos.client.config.listener.impl.AbstractConfigChangeListener;
import com.qhtx.eta.common.utils.SpringApplicationUtils;
import com.qhtx.eta.domain.config.ScheduleConfig;
import com.qhtx.eta.domain.config.ScheduleTaskConfig;
import com.qhtx.eta.domain.enums.ETATaskEnum;
import com.qhtx.eta.domain.job.ETASyncJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


@Component
@Slf4j
public class TaskConfigChangeListener extends AbstractConfigChangeListener {

    @Resource
    private NacosConfigManager nacosConfigManager;

    @Resource
    private NacosConfigProperties nacosConfigProperties;

    @Resource
    SpringApplicationUtils SpringApplicationUtils;
    @Resource
    private ETASyncJob etadataSyncJob;

    @Resource
    private FixedDelayTaskRegistrar fixedDelayTaskRegistrar;


    @Resource
    private ScheduleTaskConfig scheduleTaskConfig;

    @PostConstruct
    public void init() throws NacosException {
        String dataId = String.format("%s-%s.%s", nacosConfigProperties.getPrefix(), nacosConfigProperties.getEnvironment().getActiveProfiles()[0], nacosConfigProperties.getFileExtension());
        nacosConfigManager.getConfigService().addListener(dataId, "DATABASE_GROUP", this);
    }

    private void resetScheduleTask(ConfigChangeEvent configChangeEvent) {
        Map<Integer, List<ConfigChangeItem>> taskMap = configChangeEvent.getChangeItems().stream().filter(changeItem -> changeItem.getKey().contains("eta.scheduled.list")).collect(Collectors.groupingBy(changeItem -> {
            String indexStr = changeItem.getKey().substring(changeItem.getKey().indexOf("[") + 1, changeItem.getKey().indexOf("]"));
            try {
                return Integer.parseInt(indexStr);
            } catch (NumberFormatException e) {
                log.warn("获取任务编号失败,{}", e.getMessage());
                return -1;
            }
        }));
        //重置无效的编号
        taskMap.keySet().removeIf(key -> key < 0 || key >= scheduleTaskConfig.getList().size());
        taskMap.forEach((key, taskConfList) -> {
            if (!CollectionUtils.isEmpty(taskConfList)) {
                ScheduleConfig scheduleConfig = scheduleTaskConfig.getList().get(key);
                ETATaskEnum etaTaskEnum = etadataSyncJob.getTaskType(key);
                taskConfList.forEach(changeItem -> {
                    if (changeItem.getKey().endsWith("interval")) {
                        scheduleConfig.setInterval(Integer.valueOf(changeItem.getNewValue()));
                    }
                    if (changeItem.getKey().endsWith("initialDelay")) {
                        scheduleConfig.setInitialDelay(Integer.valueOf(changeItem.getNewValue()));
                    }
                    if (changeItem.getKey().endsWith("enabled")) {
                        scheduleConfig.setEnabled(Boolean.valueOf(changeItem.getNewValue()));
                    }
                });
                if (scheduleConfig.getEnabled()) {
                    fixedDelayTaskRegistrar.removeFixedDelayTask(new SchedulingRunnable(etadataSyncJob.getClass().getName(), etaTaskEnum.getMethod(), etadataSyncJob));
                    fixedDelayTaskRegistrar.addFixedDelayTask(new SchedulingRunnable(etadataSyncJob.getClass().getName(), etaTaskEnum.getMethod(), etadataSyncJob), scheduleConfig.getInterval(), scheduleConfig.getInitialDelay());
                } else {
                    fixedDelayTaskRegistrar.removeFixedDelayTask(new SchedulingRunnable(etadataSyncJob.getClass().getName(), etaTaskEnum.getMethod(), etadataSyncJob));
                }
            }
        });
    }

    @Override
    public void receiveConfigChange(ConfigChangeEvent configChangeEvent) {
        //            System.out.println(changeItem.getKey());
        //            if (changeItem.getKey().startsWith("eta.scheduled.info")) return "info";
        //            else if (changeItem.getKey().startsWith("eta.scheduled.list")) return "data";
        //任务相关的操作需要重置任务
        resetScheduleTask(configChangeEvent);
    }

}

