package com.qhtx.eta.infra.datasource;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.qhtx.eta.infra.utils.DruidEncryptUtil;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.beans.factory.config.BeanPostProcessor;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.Properties;
import java.util.StringTokenizer;


@Component
public class DecryptDataSourcePostProcessor implements BeanPostProcessor {
    @Value("${spring.datasource.publicKey}")
    private String publicKey;

    @Override
    public Object postProcessAfterInitialization(Object bean, String beanName) throws BeansException {
        if (bean instanceof DruidDataSource) {
            DruidDataSource dataSource = (DruidDataSource) bean;
            Properties properties = dataSource.getConnectProperties();
            Object decrypt = properties.get("decrypt");
            if (dataSource.getPassword() != null && isEncrypted(decrypt)) {
                try {
                    String decryptedPassword = DruidEncryptUtil.decrypt(publicKey, dataSource.getPassword());
                    dataSource.setPassword(decryptedPassword);
                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
            }
        }
        return bean;
    }

    private boolean isEncrypted(Object decrypt) {
        if (decrypt == null) {
            return false;
        }
        return Boolean.parseBoolean(String.valueOf(decrypt));
    }
}