package com.qhtx.eta.infra.service.impl;

import com.qhtx.eta.common.constant.ETAConstants;
import com.qhtx.eta.common.eunms.DataSourceType;
import com.qhtx.eta.common.eunms.ErrorEnum;
import com.qhtx.eta.common.exception.ETAException;
import com.qhtx.eta.infra.annotation.UseDataSource;
import com.qhtx.eta.infra.entity.dw.TDampDwIndexData;
import com.qhtx.eta.infra.mapper.TDampDwIndexDataDao;
import com.qhtx.eta.infra.service.TDampDwIndexDataService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.time.DateUtils;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionSynchronizationManager;

import javax.annotation.Resource;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.Date;
import java.util.List;

/**
 * 指标数据表(TDampDwIndexData)表服务实现类
 *
 * @author makejava
 * @since 2024-09-29 17:37:18
 */
@Service("tDampDwIndexDataService")
@UseDataSource(dataSourceType = DataSourceType.DW)
@Slf4j
public class TDampDwIndexDataServiceImpl implements TDampDwIndexDataService {
    @Resource
    private TDampDwIndexDataDao tDampDwIndexDataDao;

    @Resource
    private SqlSessionTemplate sqlSessionTemplate;

    /**
     * @param indexCode
     * @param offset
     * @param limit
     * @return
     */
    @Override
    public List<TDampDwIndexData> queryAllByLimit(String indexCode, int offset, int limit) {
        return tDampDwIndexDataDao.queryAllByLimit(indexCode, offset, limit);
    }

    @Override
    public int countByIndexCode(String indexCode) {
        return tDampDwIndexDataDao.countByIndexCode(indexCode);
    }

    @Override
    public List<TDampDwIndexData> queryByDate(String indexCode, String beginDate, int offset, int limit) {
        try {
            Date begin = DateUtils.parseDate(beginDate, "yyyy-MM-dd");
            return tDampDwIndexDataDao.queryByDate(indexCode, begin, offset, limit);
        } catch (ParseException e) {
            log.error("日期解析失败", e);
            throw new ETAException(ErrorEnum.PARSE_TIME_ERROR, "停止更新当前指标" + indexCode);
        }

    }

    @Override
    public int countByIndexCodeWithDate(String indexCode, String beginDate) {
        try {
            Date begin = DateUtils.parseDate(beginDate, "yyyy-MM-dd");
            return tDampDwIndexDataDao.countByIndexCodeWithDate(indexCode, begin);
        } catch (ParseException e) {
            log.error("日期解析失败", e);
            throw new ETAException(ErrorEnum.PARSE_TIME_ERROR, "停止更新当前指标" + indexCode);
        }
    }

    @Override
    public void batchInsertIndexData(List<TDampDwIndexData> dwLinkDataList) {
        try (SqlSession sqlSession = sqlSessionTemplate.getSqlSessionFactory().openSession(ExecutorType.BATCH, false)) {
            int index = 1;
            for (TDampDwIndexData dwLinkData : dwLinkDataList) {
                tDampDwIndexDataDao.insert(dwLinkData);
                if (index % ETAConstants.INDEX_SYNC_LIMIT == 0) {
                    sqlSession.flushStatements();
                }
                index++;
            }
            sqlSession.commit(!TransactionSynchronizationManager.isSynchronizationActive());
        } catch (Exception e) {
            log.error("批量插入指标数据失败,数据库异常", e);
            throw new ETAException(ErrorEnum.DB_ERROR);
        }
    }
}
